@@ -15,7 +15,7 @@ use std::{
 
 use pearl::{Builder, Key, Storage};
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub struct KeyTest(Vec<u8>);
 
 impl AsRef<[u8]> for KeyTest {
@@ -34,6 +34,19 @@ impl Key for KeyTest {
     const LEN: u16 = 4;
 }
 
+impl Default for KeyTest {
+    fn default() -> Self {
+        Self(vec![0; 4])
+    }
+}
+
+impl From<Vec<u8>> for KeyTest {
+    fn from(mut v: Vec<u8>) -> Self {
+        v.resize(KeyTest::LEN as usize, 0);
+        Self(v)
+    }
+}
+
 impl KeyTest {
     pub fn new(inner: u32) -> Self {
         Self(inner.to_be_bytes().to_vec())
@@ -112,6 +125,12 @@ pub async fn clean(storage: Storage<KeyTest>, path: impl AsRef<Path>) -> Result<
     fs::remove_dir_all(path).map_err(Into::into)
 }
 
+pub async fn close_storage(storage: Storage<KeyTest>) -> Result<()> {
+    std::thread::sleep(std::time::Duration::from_millis(100));
+    storage.close().await?;
+    Ok(())
+}
+
 pub async fn check_all_written(storage: &Storage<KeyTest>, keys: Vec<u32>) -> Result<(), String> {
     let mut read_futures: FuturesUnordered<_> = keys
         .iter()
