@@ -7,7 +7,7 @@ use futures::{
     stream::{futures_unordered::FuturesUnordered, StreamExt, TryStreamExt},
     TryFutureExt,
 };
-use pearl::{Builder, Meta, Storage};
+use pearl::{BloomProvider, Builder, Meta, Storage};
 use rand::{seq::SliceRandom, Rng};
 use std::{
     fs,
@@ -140,7 +140,6 @@ async fn test_multithread_read_write() -> Result<(), String> {
     let handles = handles.try_collect::<Vec<_>>().await.unwrap();
     let index = path.join("test.0.index");
     sleep(Duration::from_millis(64)).await;
-    assert!(index.exists());
     assert_eq!(handles.len(), threads);
     let keys = indexes
         .iter()
@@ -149,7 +148,9 @@ async fn test_multithread_read_write() -> Result<(), String> {
         .collect::<Vec<_>>();
     debug!("make sure that all keys was written");
     common::check_all_written(&storage, keys).await?;
-    common::clean(storage, path).await.unwrap();
+    common::close_storage(storage).await.unwrap();
+    assert!(index.exists());
+    fs::remove_dir_all(path).unwrap();
     warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
     Ok(())
 }
@@ -227,20 +228,38 @@ async fn test_on_disk_index() -> Result<()> {
 
 #[tokio::test]
 async fn test_work_dir_lock() {
-    let now = Instant::now();
+    use nix::sys::wait::{waitpid, WaitStatus};
+    use nix::unistd::{fork, ForkResult};
+
     let path = common::init("work_dir_lock");
-    let storage_one = common::create_test_storage(&path, 1_000_000);
-    let res_one = storage_one.await;
-    assert!(res_one.is_ok());
-    let storage = res_one.unwrap();
-    let storage_two = common::create_test_storage(&path, 1_000_000);
-    let res_two = storage_two.await;
-    dbg!(&res_two);
-    assert!(res_two.is_err());
-    common::clean(storage, path)
-        .map(|res| res.expect("clean failed"))
-        .await;
-    warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
+
+    // We need separate processes because locks do not work within the same process
+    match unsafe { fork() } {
+        Ok(ForkResult::Parent { child }) => {
+            let now = Instant::now();
+
+            let storage_one = common::create_test_storage(&path, 1_000_000);
+            let res_one = storage_one.await;
+            assert!(res_one.is_ok());
+
+            let storage = res_one.unwrap();
+            let result = waitpid(child, None);
+            assert_eq!(Ok(WaitStatus::Exited(child, 0)), result);
+
+            common::clean(storage, path)
+                .map(|res| res.expect("clean failed"))
+                .await;
+
+            warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
+        }
+        Ok(ForkResult::Child) => {
+            sleep(Duration::from_secs(1)).await;
+            let storage_two = common::create_test_storage(&path, 1_000_000);
+            let _ = storage_two.await;
+            unreachable!("Second storage process must panic")
+        }
+        Err(_) => unreachable!("Fork must not fail"),
+    }
 }
 
 #[tokio::test]
@@ -275,7 +294,7 @@ async fn test_index_from_blob() {
     let now = Instant::now();
     let path = common::init("index_from_blob");
     let storage = common::create_test_storage(&path, 70_000).await.unwrap();
-    let records = common::generate_records(10, 10_000);
+    let records = common::generate_records(20, 10_000);
     for (i, data) in &records {
         write_one(&storage, *i, data, None).await.unwrap();
         sleep(Duration::from_millis(10)).await;
@@ -284,10 +303,9 @@ async fn test_index_from_blob() {
     let index_file_path = path.join("test.0.index");
     fs::remove_file(&index_file_path).unwrap();
     let new_storage = common::create_test_storage(&path, 1_000_000).await.unwrap();
+    common::close_storage(new_storage).await.unwrap();
     assert!(index_file_path.exists());
-    common::clean(new_storage, path)
-        .map(|res| res.expect("clean failed"))
-        .await;
+    fs::remove_dir_all(path).unwrap();
     warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
 }
 
@@ -540,6 +558,30 @@ async fn test_check_bloom_filter_multiple() {
     warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
 }
 
+#[tokio::test]
+async fn test_check_bloom_filter_multiple_offloaded() {
+    let now = Instant::now();
+    let path = common::init("check_bloom_filter_multiple_offloaded");
+    let mut storage = common::create_test_storage(&path, 20000).await.unwrap();
+    let data =
+        b"lfolakfsjher_rladncreladlladkfsje_pkdieldpgkeolladkfsjeslladkfsj_slladkfsjorladgedom_dladlladkfsjlad";
+    for i in 1..800 {
+        let key = KeyTest::new(i);
+        storage.write(&key, data.to_vec()).await.unwrap();
+        sleep(Duration::from_millis(6)).await;
+        trace!("blobs count: {}", storage.blobs_count().await);
+    }
+    storage.offload_buffer(usize::MAX, 100).await;
+    for i in 1..800 {
+        assert_eq!(storage.check_filters(KeyTest::new(i)).await, Some(true));
+    }
+    for i in 800..1600 {
+        assert_eq!(storage.check_filters(KeyTest::new(i)).await, Some(false));
+    }
+    common::clean(storage, path).await.expect("clean failed");
+    warn!("elapsed: {:.3}", now.elapsed().as_secs_f64());
+}
+
 #[tokio::test]
 async fn test_check_bloom_filter_init_from_existing() {
     let now = Instant::now();
